<div class="form-group">
  {% if field.label %}
    <label class="sr-only">
      {{ field.label }}
    </label>
  {% endif %}

  <p class="form-control-static">Dictionaries are not currently supported in HTML input.</p>
</div>
