<div class="form-group {% if field.errors %}has-error{% endif %}">
  <div class="checkbox">
    <label>
      <input type="checkbox" name="{{ field.name }}" value="true" {% if field.value %}checked{% endif %}>
        {% if field.label %}{{ field.label }}{% endif %}
    </label>
  </div>

  {% if field.errors %}
    {% for error in field.errors %}
      <span class="help-block">{{ error }}</span>
    {% endfor %}
  {% endif %}

  {% if field.help_text %}
    <span class="help-block">{{ field.help_text|safe }}</span>
  {% endif %}
</div>
